@@ -1,7 +1,7 @@
 /* 
    Project: DataBasin
 
-   Copyright (C) 2008-2010 Free Software Foundation
+   Copyright (C) 2008-2011 Free Software Foundation
 
    Author: Riccardo Mottola
 
@@ -27,6 +27,7 @@
 #import <AppKit/AppKit.h>
 
 #import "DBSoap.h"
+#import "DBObjectInspector.h"
 
 @interface AppController : NSObject
 {
@@ -97,6 +98,9 @@
   IBOutlet NSWindow      *winDelete;
   IBOutlet NSTextField   *fieldFileDelete;
   IBOutlet NSButton      *checkSkipFirstLine;
+  
+  /* object inspector */
+  DBObjectInspector *objInspector;
 }
 
 + (void)initialize;
@@ -121,6 +125,8 @@
 - (IBAction)showSessionInspector:(id)sender;
 - (IBAction)showUserInspector:(id)sender;
 
+- (IBAction)runDescribeGlobal:(id)sender;
+
 - (IBAction)showSelect:(id)sender;
 - (IBAction)browseFileSelect:(id)sender;
 - (IBAction)executeSelect:(id)sender;
@@ -144,4 +150,6 @@
 - (IBAction)browseFileDelete:(id)sender;
 - (IBAction)executeDelete:(id)sender;
 
+- (IBAction)showObjectInspector:(id)sender;
+
 @end
