@@ -1,7 +1,7 @@
 /* 
    Project: DataBasin
 
-   Copyright (C) 2008-2010 Free Software Foundation
+   Copyright (C) 2008-2011 Free Software Foundation
 
    Author: Riccardo Mottola
 
@@ -69,12 +69,12 @@
 
 - (void)awakeFromNib
 {
-  [[NSApp mainMenu] setTitle:@"DataBasin"];
-
 #if defined(__APPLE__) && (MAC_OS_X_VERSION_MAX_ALLOWED <= MAC_OS_X_VERSION_10_4)
   [popupStrEncoding setAutoenablesItems: NO];
   [[popupStrEncoding itemAtIndex: 1] setEnabled: NO];
 #endif
+  
+  objInspector = [[DBObjectInspector alloc] init];
 }
 
 - (void)applicationDidFinishLaunching:(NSNotification *)aNotif
@@ -194,9 +194,9 @@
   db = [[DBSoap alloc] init];
   
   if ([popupEnvironment indexOfSelectedItem] == DB_ENVIRONMENT_PRODUCTION)
-    urlStr = @"http://www.salesforce.com/services/Soap/u/16.0";
+    urlStr = @"http://www.salesforce.com/services/Soap/u/20.0";
   else if ([popupEnvironment indexOfSelectedItem] == DB_ENVIRONMENT_SANDBOX)
-    urlStr = @"http://test.salesforce.com/services/Soap/u/16.0";
+    urlStr = @"http://test.salesforce.com/services/Soap/u/20.0";
   NSLog(@"Url: %@", urlStr);  
   url = [NSURL URLWithString:urlStr];
   
@@ -229,7 +229,21 @@
         [faultPanel makeKeyAndOrderFront:nil];
       }
   NS_ENDHANDLER
-    
+}
+
+/* UPDATE SOBJECT LIST */
+
+- (IBAction)runDescribeGlobal:(id)sender
+{
+  NS_DURING
+    [db updateObjects];
+  NS_HANDLER
+    if ([[localException name] hasPrefix:@"DB"])
+      {
+        [faultTextView setString:[localException reason]];
+        [faultPanel makeKeyAndOrderFront:nil];
+      }
+  NS_ENDHANDLER
 }
 
 /*  SELECT */
@@ -293,20 +307,11 @@
 
 - (IBAction)showInsert:(id)sender
 {
-  NSArray      *objectsArray;
+  NSArray *objectNames;
 
-  objectsArray  = nil;
-  NS_DURING
-    objectsArray = [db describeGlobal];
-  NS_HANDLER
-    if ([[localException name] hasPrefix:@"DB"])
-      {
-        [faultTextView setString:[localException reason]];
-        [faultPanel makeKeyAndOrderFront:nil];
-      }
-  NS_ENDHANDLER
+  objectNames = [db sObjectNames];
   [popupObjectsInsert removeAllItems];
-  [popupObjectsInsert addItemsWithTitles: objectsArray];
+  [popupObjectsInsert addItemsWithTitles: objectNames];
 
   [winInsert makeKeyAndOrderFront:self];
 }
@@ -426,20 +431,11 @@
 
 - (IBAction)showDescribe:(id)sender
 {
-  NSArray      *objectsArray;
-  
-  objectsArray  = nil;
-  NS_DURING
-    objectsArray = [db describeGlobal];
-  NS_HANDLER
-    if ([[localException name] hasPrefix:@"DB"])
-      {
-        [faultTextView setString:[localException reason]];
-        [faultPanel makeKeyAndOrderFront:nil];
-      }
-  NS_ENDHANDLER
+  NSArray *objectNames;
+
+  objectNames = [db sObjectNames];
   [popupObjectsDescribe removeAllItems];
-  [popupObjectsDescribe addItemsWithTitles: objectsArray];
+  [popupObjectsDescribe addItemsWithTitles: objectNames];
     
   [winDescribe makeKeyAndOrderFront:self];
 }
@@ -616,5 +612,12 @@
   [reader release];
 }
 
+/* OBJECT INSPECTOR */
+
+- (IBAction)showObjectInspector:(id)sender
+{
+  [objInspector setSoapHandler: db];
+  [objInspector show];
+}
 
 @end
