@@ -35,18 +35,40 @@
       qualifier = @"\"";
       separator = @",";
       newLine = @"\n";
-      encoding = NSUTF8StringEncoding;
+      [self setStringEncoding: NSUTF8StringEncoding];
+      //      encoding = NSUTF8StringEncoding;
+      //      bomLength = 3;
    }
   return self;
 }
 
 - (void)setStringEncoding: (NSStringEncoding) enc
 {
+  NSData *tempData;
+
   encoding = enc;
+  bomLength = 0;
+
+  /* BOM heuristics */
+  tempData = [@" "dataUsingEncoding: encoding];
+#if defined(__APPLE__) && (MAC_OS_X_VERSION_MAX_ALLOWED <= MAC_OS_X_VERSION_10_4)
+  NSString *blankString;
+    
+  blankString = [[[NSString alloc] initWithBytes: [tempData bytes] length: [tempData length] encoding: encoding] autorelease];
+  NSLog(@"blank string: %@", blankString);
+  tempData = [tempData subdataWithRange: NSMakeRange(0, [tempData length] - [blankString length])];
+#else
+  tempData = [tempData subdataWithRange: NSMakeRange(0, [tempData length] - [@" " lengthOfBytesUsingEncoding: encoding])];
+#endif
+  bomLength = [tempData length];
+
+  NSLog(@"bom length: %u", bomLength);
 }
 
 - (void)setFieldNames:(NSArray *)array andWriteIt:(BOOL)flag
 {
+  /* if we write the header, fine, else we write at least the BOM */
+
   if (flag == YES)
     {
       NSString *theLine;
@@ -55,6 +77,14 @@
       theLine = [self formatOneLine:array];
       [file writeData: [theLine dataUsingEncoding: encoding]];
     }
+  else
+    {
+      NSData *tempData;
+
+      tempData = [@" "dataUsingEncoding: encoding];
+      tempData = [tempData subdataWithRange: NSMakeRange(0, bomLength)];
+      [file writeData: tempData];
+    }
 }
 
 - (void)writeDataSet:(NSArray *)array
@@ -66,9 +96,16 @@
   for (i = 0; i < setCount; i++)
     {
       NSString *oneLine;
+      NSData *data;
+      NSData *data2;
       
       oneLine = [self formatOneLine:[array objectAtIndex:i]];
-      [file writeData: [oneLine dataUsingEncoding: encoding]];
+      data = [oneLine dataUsingEncoding: encoding];
+      if (bomLength > 0)
+	data2 = [NSData dataWithBytesNoCopy: [data bytes]+bomLength length: [data length]-bomLength freeWhenDone: NO];
+      else
+	data2 = data;
+      [file writeData: data2];
     }
 }
 
