@@ -1,7 +1,7 @@
 /*
    Project: DataBasin
 
-   Copyright (C) 2008-2010 Free Software Foundation
+   Copyright (C) 2008-2011 Free Software Foundation
 
    Author: Riccardo Mottola
 
@@ -27,17 +27,25 @@
 #import <WebServices/WebServices.h>
 #import "DBCVSWriter.h"
 #import "DBCVSReader.h"
-#import "DBSObject.h"
+
+#define MAX_SOQL_SIZE 9000
+
+@class DBSObject;
 
 @interface DBSoap : NSObject
 {
-    GWSService *service;
+  GWSService *service;
     
-    /* salesforce.com session variables */
-    NSString     *sessionId;
-    NSString     *serverUrl;
-    BOOL         passwordExpired;
-    NSDictionary *userInfo;
+  /* salesforce.com session variables */
+  NSString     *sessionId;
+  NSString     *serverUrl;
+  BOOL         passwordExpired;
+  NSDictionary *userInfo;
+
+  /* list of all objects, custom and not */
+  NSArray  *sObjectList;
+  /* list of all object names, custom and not */
+  NSMutableArray  *sObjectNamesList;
 }
 
 - (void)login :(NSURL *)url :(NSString *)userName :(NSString *)password;
@@ -52,8 +60,12 @@
 - (NSMutableArray *)delete :(NSArray *)objectIdArray;
 - (NSMutableArray *)deleteFromReader:(DBCVSReader *)reader;
 - (NSArray *)describeGlobal;
+- (NSArray *)sObjects;
+- (NSArray *)sObjectNames;
+- (void)updateObjects;
 - (void)describeSObject: (NSString *)objectType toWriter:(DBCVSWriter *)writer;
 - (DBSObject *)describeSObject: (NSString *)objectType;
+- (NSString *)identifyObjectById:(NSString *)sfId;
 
 - (NSString *) sessionId;
 - (NSString *) serverUrl;
