@@ -1,7 +1,7 @@
 /*
    Project: DataBasin
 
-   Copyright (C) 2008-2010 Free Software Foundation
+   Copyright (C) 2008-2011 Free Software Foundation
 
    Author: Riccardo Mottola
 
@@ -25,7 +25,7 @@
 #import <AppKit/AppKit.h>
 
 #import "DBSoap.h"
-
+#import "DBSObject.h"
 
 
 @implementation DBSoap
@@ -50,8 +50,8 @@
   [defs registerDefaults:
     [NSDictionary dictionaryWithObjectsAndKeys:
       @"80", @"Port",
-      nil]
-    ];
+      nil]]
+    ;
 
   /* initialize the coder */
   coder = [GWSSOAPCoder new];
@@ -168,8 +168,8 @@
     NSLog(@"serverUrl: %@", serverUrl);
   }
   
-  [service setURL:serverUrl];
-}
+  [service setURL:serverUrl];}
+
 
 /** <p>Execute SOQL query <i>queryString</i> and returns the resulting DBSObjects as an array.</p>
   <p>This method will query all resultinng objects of the query, repeatedly querying again if necessary depending on the batch size.</p>
@@ -186,8 +186,8 @@
   while (qLoc != nil)
     qLoc = [self queryMore: qLoc toArray: sObjects];
   
-  return sObjects;
-}
+  return sObjects;}
+
 
 /** <p>execute SOQL query and write the resulting DBSObjectes into the <i>objects</i> array
   which must be valid and allocated. </p>
@@ -253,7 +253,7 @@
 				  order : nil
 				timeout : 90];
     }
-
+  NSLog(@"result: %@", resultDict);
   queryFault = [resultDict objectForKey:@"GWSCoderFault"];
   if (queryFault != nil)
     {
@@ -264,12 +264,12 @@
       fault = [faultDetail objectForKey:@"fault"];
       NSLog(@"fault: %@", fault);
       NSLog(@"exception code: %@", [fault objectForKey:@"exceptionCode"]);
-      NSLog(@"exception code: %@", [fault objectForKey:@"exceptionMessage"]);
+      NSLog(@"exception: %@", [fault objectForKey:@"exceptionMessage"]);
     }
   
   queryResult = [resultDict objectForKey:@"GWSCoderParameters"];
   result = [queryResult objectForKey:@"result"];
-  
+  NSLog(@"result: %@", result);  
   doneStr = [result objectForKey:@"done"];
   records = [result objectForKey:@"records"];
   sizeStr = [result objectForKey:@"size"];
@@ -356,8 +356,8 @@
       queryLocator = [result objectForKey:@"queryLocator"];
       NSLog(@"should do query more, queryLocator: %@", queryLocator);
     }
-  return queryLocator;
-}
+  return queryLocator;}
+
 
 /** <p>Execute SOQL query more and write the resulting DBSObjectes into the <i>objects</i> array
     which must be valid and allocated, continuing from the given query locator <i>locator</i>. </p>
@@ -514,8 +514,8 @@
       queryLocator = [result objectForKey:@"queryLocator"];
       NSLog(@"should do query more, queryLocator: %@", queryLocator);
     }
-  return queryLocator;
-}
+  return queryLocator;}
+
 
 - (void)query :(NSString *)queryString queryAll:(BOOL)all toWriter:(DBCVSWriter *)writer
 {
@@ -586,8 +586,8 @@
       [writer writeDataSet:set];
       [set release];
       
-    }
-}
+    }}
+
 
 
 - (void)create :(NSString *)objectName fromReader:(DBCVSReader *)reader
@@ -759,8 +759,8 @@
         }
       /* we don't do yet anything useful with the results... */
       [set release];
-    }
-}
+    }}
+
 
 - (void)update :(NSString *)objectName fromReader:(DBCVSReader *)reader
 {
@@ -932,10 +932,11 @@
       }
       /* we don't do yet anything useful with the results... */
       [set release];
-    }
-}
+    }}
 
 
+
+/** runs a describe global to retrieve all all the objects and returns an array of DBSobjects */
 - (NSArray *)describeGlobal
 {
   NSMutableDictionary   *headerDict;
@@ -946,7 +947,9 @@
   NSDictionary          *queryResult;
   NSDictionary          *result;
   NSDictionary          *queryFault;
-  NSArray               *types;
+  NSArray               *sobjects;
+  NSMutableArray        *objectList;
+  int                   i;
 
   /* prepare the header */
   sessionHeaderDict = [NSMutableDictionary dictionaryWithCapacity: 2];
@@ -973,7 +976,7 @@
 		timeout : 90];
 
 
-  NSLog(@"dict is %d big", [resultDict count]);
+  NSLog(@"Describe Global dict is %d big", [resultDict count]);
   
   queryFault = [resultDict objectForKey:@"GWSCoderFault"];
   if (queryFault != nil)
@@ -991,11 +994,79 @@
 
   queryResult = [resultDict objectForKey:@"GWSCoderParameters"];
   result = [queryResult objectForKey:@"result"];
-  NSLog(@"result: %@", result);
+//  NSLog(@"result: %@", result);
 
-  types = [result objectForKey:@"types"];
+  objectList = [NSMutableArray arrayWithCapacity:1];
+  sobjects = [result objectForKey:@"sobjects"];
 
-  return types;
+  for (i = 0; i < [sobjects count]; i++)
+    {
+      NSMutableDictionary *sObj;
+      NSArray *propertiesArray;
+      NSMutableDictionary *propertiesDict;
+      DBSObject *dbObj;
+      int j;
+    
+      sObj = [sobjects objectAtIndex: i];
+      propertiesArray = [sObj objectForKey: @"GWSCoderOrder"];
+      propertiesDict = [NSMutableDictionary dictionaryWithCapacity: [propertiesArray count]];
+      for (j = 0; j < [propertiesArray count]; j++)
+	{
+	  NSString *key;
+	  
+	  key = [propertiesArray objectAtIndex:j];
+	  [propertiesDict setObject: [sObj objectForKey: key] forKey: key];
+	}
+      dbObj = [[DBSObject alloc] init];
+      [dbObj setObjectProperties: propertiesDict];
+      [objectList addObject: dbObj];
+      [dbObj release];
+    }
+
+  return [NSArray arrayWithArray: objectList];
+}
+
+/* returns the currently stored list of object names
+   if the list is nil, a describe global will be run to obtain it */
+- (NSArray *)sObjects
+{
+  if (sObjectList == nil)
+    sObjectList = [[self describeGlobal] retain];
+
+  return sObjectList;
+}
+
+/* returns the currently stored list of object names
+   if the list is nil, a describe global will be run to obtain it */
+- (NSArray *)sObjectNames
+{
+  if (sObjectNamesList == nil)
+    {
+      int i;
+
+      if (sObjectList == nil)
+	sObjectList = [[self describeGlobal] retain];
+
+      sObjectNamesList = [[NSMutableArray arrayWithCapacity:1] retain];
+      for (i = 0; i < [sObjectList count]; i++)
+	[sObjectNamesList addObject: [[sObjectList objectAtIndex: i] name]];
+    }
+  
+  return sObjectNamesList;
+}
+
+/** Force an udpate to the currently stored object  list */
+- (void)updateObjects
+{
+  int i;
+
+  [sObjectList release];
+  sObjectList = [[self describeGlobal] retain];
+
+  [sObjectNamesList release];
+  sObjectNamesList = [[NSMutableArray arrayWithCapacity:1] retain];
+  for (i = 0; i < [sObjectList count]; i++)
+    [sObjectNamesList addObject: [[sObjectList objectAtIndex: i] name]];
 }
 
 - (void)describeSObject: (NSString *)objectType toWriter:(DBCVSWriter *)writer
@@ -1064,6 +1135,7 @@
   int                   size;
   NSMutableArray        *keys;
   DBSObject             *object;
+  NSMutableDictionary   *propDict;
 
 
   /* prepare the header */
@@ -1122,6 +1194,9 @@
   [keys removeObject:@"GWSCoderOrder"];
 
   object = [[DBSObject alloc] init];
+  propDict = [NSMutableDictionary dictionaryWithCapacity: 1];
+  [propDict setValue: objectType forKey: @"name"];
+  [object setObjectProperties: propDict];
 
   for (i = 0; i < size; i++)
     {
@@ -1269,6 +1344,52 @@
   return resultArray;
 }
 
+- (NSString *)identifyObjectById:(NSString *)sfId
+{
+  NSString *devName;
+  NSEnumerator *enu;
+  NSString *name;
+  BOOL found;
+  NSString *prefixToIdentify;
+  DBSObject *tempObj;
+
+  devName = nil;
+  found = NO;
+
+  if (sfId == nil)
+    return nil;
+
+  if (!([sfId length] == 15 || [sfId length] == 18))
+    return nil;
+
+  prefixToIdentify = [sfId substringToIndex: 3];
+  NSLog(@"identify: %@", prefixToIdentify);
+  if (sObjectList == nil)
+    [self updateObjects];
+
+  NSLog(@"in %d objects", [sObjectList count]);
+  enu = [sObjectList objectEnumerator];
+  while (!found && (tempObj = [enu nextObject]))
+    {
+      NSLog(@"compare to: %@", [[tempObj objectProperties] objectForKey: @"keyPrefix"]);
+      if ([[[tempObj objectProperties] objectForKey: @"keyPrefix"] isEqualToString: prefixToIdentify])
+	{
+	  name = [tempObj name];
+	  NSLog(@"we have something: %@", name);
+	  found = YES;
+	}
+    }
+
+  if (found)
+    {
+      NSLog(@"we found: %@", name);
+      devName = [NSString stringWithString: name];
+    }
+  else
+    NSLog(@"not found");
+  return devName;
+}
+
 /* accessors*/
 - (NSString *) sessionId
 {
